#include <tivo-timing.h>
#include <fcntl.h>
#include <string.h>

#if TV_TIMING_AVAIL
/* Read cycle counter frequency from TiVo /proc/cpuinfo entry */
const static char cycle_counter_str[] = "cycle counter frequency";

void
__tv_get_clockfreq(unsigned long long *freq_ret) {
  static unsigned long long freq = 0;
  int fd;

  /* value may be cached. In order to be MT-safe, this function must
     be called for the first time before any threads have been spun */
  if (freq != 0) { /* value is cached */
    *freq_ret = freq;
    return;
  }

  fd = open ("/proc/cpuinfo", O_RDONLY);
  if (fd != -1) {
    char buf[4096];  /* supposed limit of cpuinfo */
    int  n;

    n = read(fd, buf, sizeof(buf));
    if (n > 0) {
      unsigned long long result = 0;
      char *freqstr = memmem (buf, n, cycle_counter_str, 
                              sizeof(cycle_counter_str) - 1);
      char *endp = buf + n;
      if (freqstr != NULL) {
        /* find first digit */
        while (freqstr < endp && *freqstr != '\n' &&
               ( (*freqstr <  '0') || (*freqstr >  '9'))) {
	    freqstr++;
        }

        while (freqstr < endp && *freqstr != '\n' &&
               ( (*freqstr >= '0') && (*freqstr <= '9'))) {
            result = result * 10;
            result += (*freqstr++ - '0');
        }
      }
      /* if 64bit assign is not atomic, this assignment is not thread safe */
      freq = result;
    }
    close(fd);
  }
  *freq_ret = freq;
  return;
}
#else
void
__tv_get_clockfreq(unsigned long long *freq_ret) {}
#endif




